import time

from .channel_interface import ChannelInterface
import logging
from OpenHub.globals import id_hardware_map
import asyncio


class AM2315Humidity(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, channel_stats=None,*args, **kwargs):
        self.am2315 = id_hardware_map[config['hardware']]
        self.serial_no = config['id']
        self.type = config['type']
        super().__init__(config=config,hardware_serial_no=hardware_serial_no, serial_no=self.serial_no, channel_stats=channel_stats,*args, **kwargs)

    async def get_raw_data(self):
        humidity = None
        while humidity is None:
            try:
                humidity = self.am2315.get_humidity()
            except Exception as e:
                self.logger.info(str(e))
                await asyncio.sleep(0.01)
        return {'value':humidity}



import json
import logging
import RPi.GPIO as GPIO
from serial import Serial
from .channels.pi_pico_pump import PiPicoPump
from .channels.pi_pico_analog import PiPicoAnalog
from .channels.pi_pico_relay import PiPicoRelay
from .hardware_interface import HardwareInterface

logger = logging.getLogger(__name__)
from asyncio import Lock


class Pi(HardwareInterface):
    lock = Lock()

    serial_no = None

    serial = None

    channels = []

    def __init__(self, config, channels=None, *args, **kwargs):
        self.config = config

        # self.channels_config = pico_config['channels']
        self.serial_no = config['id']
        super(Pi, self).__init__(self.serial_no,  *args, **kwargs)

    @classmethod
    def _gpio_setup(_cls):
        if GPIO.getmode() is None:
            GPIO.setmode(GPIO.BOARD)

    @classmethod
    def create_output_pin(_cls, pin):
        Pi._gpio_setup()
        GPIO.setup(pin, GPIO.OUT)

    @classmethod
    def create_input_pin(_cls, pin):
        Pi._gpio_setup()
        GPIO.setup(pin, GPIO.IN)


import logging
from OpenHub.hardware_interfaces import hardware_interface
from OpenHub.hardware_interfaces.channels.dht22_humidity import DHT22Humidity
from OpenHub.hardware_interfaces.channels.dht22_temp import DHT22Temp
import board
import busio
import adafruit_am2320


class AM2315(hardware_interface.HardwareInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, scl=board.SCL, sda=board.SDA, serial_no=None, channels=None, *args, **kwargs):
        self.i2c = busio.I2C(scl, sda)
        self.am2315 = adafruit_am2320.AM2320(self.i2c)
        self.type = __name__
        super().__init__(serial_no, channels, *args, **kwargs)

    def get_temp(self):
        return self.am2315.temperature

    def get_humidity(self):
        return self.am2315.relative_humidity



from .channel_interface import ChannelInterface
import logging
from OpenHub.globals import id_hardware_map
import json
import RPi.GPIO as GPIO
from OpenHub.hardware_interfaces.pi import Pi

class PiRelay(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.serial_no = config['id']
        self.type = config['type']

        super().__init__(config=config, serial_no=self.serial_no, *args, **kwargs)

    def setup(self,pin):
        self.pin = pin
        Pi.create_output_pin(self.pin)

    async def turn_on(self):
        GPIO.output(self.pin, 1)


    async def turn_off(self):
        GPIO.output(self.pin, 0)

    async def get_raw_data(self):
        pass


import logging
from .data_transformer import DataTransformer


class Sum(DataTransformer):

    def perform_op(self, inputs):
        out = sum(inputs)
        self.logger.info('summed ' + str(out))

        return out


import logging
from .data_transformer import DataTransformer


class Inverse(DataTransformer):

    def perform_op(self, inputs):
        product = 1
        for input in inputs:
            if input == 0:
                self.logger.info('input is 0 returning -1 ')

                return -1
            product = float(1/float(input))
        self.logger.info('inverted ' + str(product))
        return product


import json
from OpenHub.hardware_interfaces.channels.dht22_humidity import DHT22Humidity
from OpenHub.hardware_interfaces.channels.dht22_temp import DHT22Temp
from OpenHub.hardware_interfaces.channels.mcp3008analog import MCP3008Analog
from OpenHub.hardware_interfaces.channels.mod_probe_temp import ModProbeTemp
from OpenHub.hardware_interfaces.channels.pi_pico_analog import PiPicoAnalog
from OpenHub.hardware_interfaces.channels.pi_pico_ac_analog import PiPicoACAnalog
from OpenHub.hardware_interfaces.channels.pi_pico_pump import PiPicoPump
from OpenHub.hardware_interfaces.channels.veml7700_light import VEML7700Light
from OpenHub.hardware_interfaces.channels.veml7700_lux import VEML7700Lux
from OpenHub.hardware_interfaces.channels.am2315_temperature import AM2315Temperature
from OpenHub.hardware_interfaces.channels.am2315_humidity import AM2315Humidity
from OpenHub.hardware_interfaces.channels.pmsa0031_25 import PMSAAirQuality25
from OpenHub.hardware_interfaces.channels.pmsa0031_100 import PMSAAirQuality100
# from OpenHub.hardware_interfaces.channels.pmsa0031 import PMSA0031

from OpenHub.hardware_interfaces.channels.pi_pico_relay import PiPicoRelay
from OpenHub.hardware_interfaces.channels.pi_relay import PiRelay
from OpenHub.hardware_interfaces.channels.stat.max import Max
from OpenHub.hardware_interfaces.channels.stat.min import Min
import logging

class ChannelDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, config):
        logger = logging.getLogger(ChannelDecoder.__name__)
        logger.info(str(config))
        model = config['model']
        type = config['type']
        if model=='ChannelStats' or model=='ChannelStat':
            if type == 'MAX':
                return Max(config)
            if type == 'MIN':
                return Min(config)

        type = config['type']
        if 'channelstats_set' in config.keys():
            stats=config['channelstats_set']
        elif 'channel_stats' in config.keys():
            stats=config['channelstats']
        else:
            stats=None

        if type == DHT22Humidity.__name__:
            return DHT22Humidity(config=config, channel_stats=stats)
        if type == DHT22Temp.__name__:
            return DHT22Temp(config=config, channel_stats=stats)
        elif type == MCP3008Analog.__name__:
            return MCP3008Analog()

        elif type == ModProbeTemp.__name__:
            return ModProbeTemp(config=config, channel_stats=stats)

        elif type == PiPicoAnalog.__name__:
            return PiPicoAnalog(config=config, channel_stats=stats)
        elif type == PiPicoACAnalog.__name__:
            return PiPicoACAnalog(config=config, channel_stats=stats)
        elif type == PiPicoPump.__name__:
            return PiPicoPump(config=config, channel_stats=stats)

        elif type == PiPicoRelay.__name__:
            return PiPicoRelay(config=config, channel_stats=stats)
        elif type == PiRelay.__name__:
            return PiRelay(config=config, channel_stats=stats)
        elif type == VEML7700Light.__name__:
            return VEML7700Light(config=config, channel_stats=stats)

        elif type == VEML7700Lux.__name__:
            return VEML7700Lux(config=config, channel_stats=stats)
        elif type == AM2315Temperature.__name__:
            return AM2315Temperature(config=config, channel_stats=stats)
        elif type == AM2315Humidity.__name__:
            return AM2315Humidity(config=config, channel_stats=stats)
        elif type == PMSAAirQuality25.__name__:
            return PMSAAirQuality25(config=config, channel_stats=stats)
        elif type == PMSAAirQuality100.__name__:
            return PMSAAirQuality100(config=config, channel_stats=stats)

        return super().object_hook( config)